@@ -6,19 +6,19 @@ use libc::{self, c_char};
 /// Given a valid C-style, NUL terminated, UTF8-encoded string, returns a Rust `&str`
 ///
 /// Warnings:
-/// - No checks are made for: NULL pointers, valid UTF-8
+/// - No checks are made for: valid UTF-8
 /// - This function "creates" a reference with an arbitrary lifetime, so be careful to limit the
 ///   lifetime appropriately
+#[inline]
 pub(crate) unsafe fn str_from_cstr_ptr<'a>(ptr: *const c_char) -> Option<&'a str> {
     if ptr.is_null() {
         return None;
     }
-
     let len = libc::strlen(ptr);
-
     /* ASSUMPTION: capstone returns NUL terminated string */
     let view: &[u8] = slice::from_raw_parts(ptr as *const u8, len as usize);
-    str::from_utf8(view).ok()
+    /* ASSUMPTION: capstone returns a valid UTF-8 string */
+    Some(str::from_utf8_unchecked(view))
 }
 
 #[cfg(test)]
@@ -30,10 +30,6 @@ mod test {
     fn cstr_convert() {
         unsafe {
             assert_eq!(str_from_cstr_ptr(core::ptr::null() as *const c_char), None);
-            assert_eq!(
-                str_from_cstr_ptr(b"\xff\x00".as_ptr() as *const c_char),
-                None
-            );
             assert_eq!(
                 str_from_cstr_ptr(b"\x00".as_ptr() as *const c_char),
                 Some("")
